@@ -2,9 +2,8 @@
 
     file                 : human.cpp
     created              : Sat Mar 18 23:16:38 CET 2000
-    copyright            : (C) 2000 by Eric Espie
-    email                : torcs@free.fr
-    version              : $Id: human.cpp,v 1.45.2.9 2012/02/09 22:36:27 berniw Exp $
+    copyright            : (C) 2000-2024 by Eric Espie, Bernhard Wymann
+    email                : berniw@bluewin.ch
 
  ***************************************************************************/
 
@@ -17,10 +16,9 @@
  *                                                                         *
  ***************************************************************************/
 
-/** @file   
-    		
-    @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: human.cpp,v 1.45.2.9 2012/02/09 22:36:27 berniw Exp $
+/** @file
+	Human driver
+	@author	Bernhard Wymann, Eric Espie
 */
 
 
@@ -59,13 +57,9 @@ static int  pitcmd(int index, tCarElt* car, tSituation *s);
 
 int joyPresent = 0;
 
-//static char	sstring[1024];
-//static char	buf[1024];
-
 static tTrack	*curTrack;
-//static void	*DrvInfo;
 
-static float color[] = {0.0, 0.0, 1.0, 1.0};
+static float color[] = {0.5, 0.5, 1.0, 1.0};
 
 static tCtrlJoyInfo	*joyInfo = NULL;
 static tCtrlMouseInfo	*mouseInfo = NULL;
@@ -90,7 +84,7 @@ static tKeyInfo skeyInfo[256];
 static int currentKey[256];
 static int currentSKey[256];
 
-static tdble lastKeyUpdate = -10.0;
+static double lastKeyUpdate = -10.0;
 
 static int	firstTime = 0;
 
@@ -105,13 +99,11 @@ BOOL WINAPI DllEntryPoint (HINSTANCE hDLL, DWORD dwReason, LPVOID Reserved)
 static void
 shutdown(int index)
 {
-	//static int	firstTime = 1;
-	int		idx = index - 1;
+	int	idx = index - 1;
 
 	free (HCtx[idx]);
 
 	if (firstTime) {
-		//GfParmReleaseHandle(DrvInfo);
 		GfParmReleaseHandle(PrefHdle);
 		GfctrlJoyRelease(joyInfo);
 		GfctrlMouseRelease(mouseInfo);
@@ -128,7 +120,7 @@ shutdown(int index)
  *	InitFuncPt
  *
  * Description
- *	Robot functions initialisation
+ *	Robot functions initialization
  *
  * Parameters
  *	pt	pointer on functions structure
@@ -151,7 +143,6 @@ InitFuncPt(int index, void *pt)
 
 	if (firstTime < 1) {
 		firstTime = 1;
-		//DrvInfo = GfParmReadFile(buf, GFPARM_RMODE_REREAD | GFPARM_RMODE_CREAT);
 		joyInfo = GfctrlJoyInit();
 		if (joyInfo) {
 			joyPresent = 1;
@@ -202,9 +193,6 @@ InitFuncPt(int index, void *pt)
  *
  */
 
-#define MAXNAMELEN 100
-
-static char names[10][MAXNAMELEN];
 
 extern "C" int
 human(tModInfo *modInfo)
@@ -227,9 +215,9 @@ human(tModInfo *modInfo)
 			if (strlen(driver) == 0) {
 				break;
 			}
-			strncpy(names[i], driver, MAXNAMELEN);
-			modInfo->name    = names[i];	/* name of the module (short) */
-			modInfo->desc    = strdup("Joystick controlable driver");	/* description of the module (can be long) */
+
+			modInfo->name    = strdup(driver);	/* name of the module (short) */
+			modInfo->desc    = strdup("Joystick controllable driver");	/* description of the module (can be long) */
 			modInfo->fctInit = InitFuncPt;	/* init function */
 			modInfo->gfId    = ROB_IDENT;	/* supported framework version */
 			modInfo->index   = i+1;
@@ -265,28 +253,15 @@ human(tModInfo *modInfo)
 static void initTrack(int index, tTrack* track, void *carHandle, void **carParmHandle, tSituation *s)
 {
 	const char *carname;
-	char *s1, *s2;
 	const int BUFSIZE = 1024;
 	char buf[BUFSIZE];
 	char sstring[BUFSIZE];
-	
-	const int TRACKNAMESIZE = 256;
-	char trackname[TRACKNAMESIZE];
 	tdble fuel;
 	int idx = index - 1;
 
 	curTrack = track;
-	s1 = strrchr(track->filename, '/') + 1;
-	s2 = strchr(s1, '.');
-	if (s2-s1 < TRACKNAMESIZE) {
-		strncpy(trackname, s1, s2-s1);
-		trackname[s2-s1] = 0;
-	} else {
-		trackname[0] = 0;
-		GfError("human.cpp, initTrack, filename too long");
-	}
-	snprintf(sstring, BUFSIZE, "Robots/index/%d", index);
 
+	snprintf(sstring, BUFSIZE, "Robots/index/%d", index);
 	snprintf(buf, BUFSIZE, "%sdrivers/human/human.xml", GetLocalDir());
 	void *DrvInfo = GfParmReadFile(buf, GFPARM_RMODE_REREAD | GFPARM_RMODE_CREAT);
 	carname = "";
@@ -294,36 +269,31 @@ static void initTrack(int index, tTrack* track, void *carHandle, void **carParmH
 		carname = GfParmGetStr(DrvInfo, sstring, "car name", "");
 	}
 
-	snprintf(sstring, BUFSIZE, "%sdrivers/human/tracks/%s/car-%s-%d.xml", GetLocalDir(), trackname, carname, index);
-	*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-	if (*carParmHandle != NULL) {
-		GfOut("Player: %s Loaded\n", sstring);
-	} else {
-		snprintf(sstring, BUFSIZE, "%sdrivers/human/tracks/%s/car-%s.xml", GetLocalDir(), trackname, carname);
-		*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-		if (*carParmHandle != NULL) {
-			GfOut("Player: %s Loaded\n", sstring);
-		} else {
-			snprintf(sstring, BUFSIZE, "%sdrivers/human/car-%s-%d.xml", GetLocalDir(), carname, index);
-			*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-			if (*carParmHandle != NULL) {
-				GfOut("Player: %s Loaded\n", sstring);
-			} else {
-				snprintf(sstring, BUFSIZE, "%sdrivers/human/car-%s.xml", GetLocalDir(), carname);
-				*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-				if (*carParmHandle != NULL) {
-					GfOut("Player: %s Loaded\n", sstring);
-				} else {
-					snprintf(sstring, BUFSIZE, "%sdrivers/human/car.xml", GetLocalDir ());
-					*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
-					if (*carParmHandle != NULL) {
-						GfOut("Player: %s Loaded\n", sstring);
-					}
-				}
-			}
-		}
+	*carParmHandle = NULL;
+	// If session type is "race" and we have a race setup use it
+	if (s->_raceType == RM_TYPE_RACE) {
+		*carParmHandle = RtParmReadSetup(RACE, "human", index, track->internalname, carname);
 	}
 
+	// If session type is "qualifying" and we have a qualifying setup use it, use qualifying setup as 
+	// fallback if not race setup is available
+	if (s->_raceType == RM_TYPE_QUALIF || (*carParmHandle == NULL && s->_raceType == RM_TYPE_RACE)) {
+		*carParmHandle = RtParmReadSetup(QUALIFYING, "human", index, track->internalname, carname);
+	}
+
+	// If we have not yet loaded a setup we have not found a fitting one or want to use the practice setup,
+	// so try to load this
+	if (*carParmHandle == NULL) {
+		*carParmHandle = RtParmReadSetup(PRACTICE, "human", index, track->internalname, carname);
+	}
+
+	// Absolute fallback, nothing found
+	if (*carParmHandle == NULL) {
+		snprintf(sstring, BUFSIZE, "%sdrivers/human/car.xml", GetLocalDir ());
+		*carParmHandle = GfParmReadFile(sstring, GFPARM_RMODE_REREAD);
+	}
+
+
 	if (curTrack->pits.type != TR_PIT_NONE) {
 		snprintf(sstring, BUFSIZE, "%s/%s/%d", HM_SECT_PREF, HM_LIST_DRV, index);
 		HCtx[idx]->NbPitStopProg = (int)GfParmGetNum(PrefHdle, sstring, HM_ATT_NBPITS, (char*)NULL, 0);
@@ -332,7 +302,9 @@ static void initTrack(int index, tTrack* track, void *carHandle, void **carParmH
 		HCtx[idx]->NbPitStopProg = 0;
 	}
 	fuel = 0.0008 * curTrack->length * (s->_totLaps + 1) / (1.0 + ((tdble)HCtx[idx]->NbPitStopProg)) + 20.0;
-	GfParmSetNum(*carParmHandle, SECT_CAR, PRM_FUEL, (char*)NULL, fuel);
+	if (*carParmHandle) {
+		GfParmSetNum(*carParmHandle, SECT_CAR, PRM_FUEL, (char*)NULL, fuel);
+	}
 	Vtarget = curTrack->pits.speedLimit;
 	if (DrvInfo != NULL) {
 		GfParmReleaseHandle(DrvInfo);
@@ -356,18 +328,8 @@ static void initTrack(int index, tTrack* track, void *carHandle, void **carParmH
 
 void newrace(int index, tCarElt* car, tSituation *s)
 {
-	int i;
 	int idx = index - 1;
 
-	for (i = 0; i < MAX_GEARS; i++) {
-		if (car->_gearRatio[i] != 0) {
-			HCtx[idx]->shiftThld[i] = car->_enginerpmRedLine * car->_wheelRadius(2) * 0.85 / car->_gearRatio[i];
-			GfOut("Gear %d: Spd %f\n", i, HCtx[idx]->shiftThld[i] * 3.6);
-		} else {
-	    	HCtx[idx]->shiftThld[i] = 10000.0;
-		}
-	}
-
 	if (HCtx[idx]->MouseControlUsed) {
 		GfctrlMouseCenter();
 	}
@@ -536,6 +498,28 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 		lastKeyUpdate = s->currentTime;
 	}
 
+
+	// Brake repartition offset, init with former value 
+	car->ctrl.brakeRepartitionCmd = car->priv.brakeRepartitionCmd;
+
+	if (((cmd[CMD_BRAKE_TOFRONT].type == GFCTRL_TYPE_JOY_BUT) && joyInfo->edgeup[cmd[CMD_BRAKE_TOFRONT].val]) ||
+		((cmd[CMD_BRAKE_TOFRONT].type == GFCTRL_TYPE_KEYBOARD) && keyInfo[cmd[CMD_BRAKE_TOFRONT].val].edgeUp) ||
+		((cmd[CMD_BRAKE_TOFRONT].type == GFCTRL_TYPE_SKEYBOARD) && skeyInfo[cmd[CMD_BRAKE_TOFRONT].val].edgeUp))
+	{
+		if (car->priv.brakeRepartitionCmd < car->priv.repCmdMaxClicks) {
+			car->ctrl.brakeRepartitionCmd = car->priv.brakeRepartitionCmd + 1;
+		}
+	}
+
+	if (((cmd[CMD_BRAKE_TOREAR].type == GFCTRL_TYPE_JOY_BUT) && joyInfo->edgeup[cmd[CMD_BRAKE_TOREAR].val]) ||
+		((cmd[CMD_BRAKE_TOREAR].type == GFCTRL_TYPE_KEYBOARD) && keyInfo[cmd[CMD_BRAKE_TOREAR].val].edgeUp) ||
+		((cmd[CMD_BRAKE_TOREAR].type == GFCTRL_TYPE_SKEYBOARD) && skeyInfo[cmd[CMD_BRAKE_TOREAR].val].edgeUp))
+	{
+		if (car->priv.brakeRepartitionCmd > -car->priv.repCmdMaxClicks) {
+			car->ctrl.brakeRepartitionCmd = car->priv.brakeRepartitionCmd - 1;
+		}
+	}
+
 	if (((cmd[CMD_ABS].type == GFCTRL_TYPE_JOY_BUT) && joyInfo->edgeup[cmd[CMD_ABS].val]) ||
 		((cmd[CMD_ABS].type == GFCTRL_TYPE_KEYBOARD) && keyInfo[cmd[CMD_ABS].val].edgeUp) ||
 		((cmd[CMD_ABS].type == GFCTRL_TYPE_SKEYBOARD) && skeyInfo[cmd[CMD_ABS].val].edgeUp))
@@ -557,7 +541,13 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 	}
 
 	const int bufsize = sizeof(car->_msgCmd[0]);
-	snprintf(car->_msgCmd[0], bufsize, "%s %s", (HCtx[idx]->ParamAbs ? "ABS" : ""), (HCtx[idx]->ParamAsr ? "ASR" : ""));
+
+	snprintf(car->_msgCmd[0], bufsize, "BIAS: %+d %s%s%s",
+		car->ctrl.brakeRepartitionCmd,
+		(HCtx[idx]->ParamAbs ? "ABS" : ""),
+		(HCtx[idx]->ParamAbs && HCtx[idx]->ParamAsr ? " " : ""),
+		(HCtx[idx]->ParamAsr ? "ASR" : ""));
+
 	memcpy(car->_msgColorCmd, color, sizeof(car->_msgColorCmd));
 
 	if (((cmd[CMD_SPDLIM].type == GFCTRL_TYPE_JOY_BUT) && (joyInfo->levelup[cmd[CMD_SPDLIM].val] == 1)) ||
@@ -594,7 +584,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 			
 			// normalize ax0 to -1..0
 			ax0 = (ax0 - cmd[CMD_LEFTSTEER].max) / (cmd[CMD_LEFTSTEER].max - cmd[CMD_LEFTSTEER].min);
-			leftSteer = -SIGN(ax0) * cmd[CMD_LEFTSTEER].pow * pow(fabs(ax0), cmd[CMD_LEFTSTEER].sens) / (1.0 + cmd[CMD_LEFTSTEER].spdSens * car->_speed_x);
+			leftSteer = -SIGN(ax0) * cmd[CMD_LEFTSTEER].pow * pow(fabs(ax0), cmd[CMD_LEFTSTEER].sens) / (1.0 + cmd[CMD_LEFTSTEER].spdSens * car->pub.speed);
 			break;
 		case GFCTRL_TYPE_MOUSE_AXIS:
 			ax0 = mouseInfo->ax[cmd[CMD_LEFTSTEER].val] - cmd[CMD_LEFTSTEER].deadZone; //FIXME: correct?
@@ -604,7 +594,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 				ax0 = cmd[CMD_LEFTSTEER].min;
 			}
 			ax0 = ax0 * cmd[CMD_LEFTSTEER].pow;
-			leftSteer = pow(fabs(ax0), cmd[CMD_LEFTSTEER].sens) / (1.0 + cmd[CMD_LEFTSTEER].spdSens * car->_speed_x / 10.0);
+			leftSteer = pow(fabs(ax0), cmd[CMD_LEFTSTEER].sens) / (1.0 + cmd[CMD_LEFTSTEER].spdSens * car->pub.speed / 10.0);
 			break;
 		case GFCTRL_TYPE_KEYBOARD:
 		case GFCTRL_TYPE_SKEYBOARD:
@@ -620,7 +610,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 				HCtx[idx]->prevLeftSteer = leftSteer = 0;
 			} else {
 				ax0 = 2 * ax0 - 1;
-				leftSteer = HCtx[idx]->prevLeftSteer + ax0 * cmd[CMD_LEFTSTEER].sens * s->deltaTime / (1.0 + cmd[CMD_LEFTSTEER].spdSens * car->_speed_x / 10.0);
+				leftSteer = HCtx[idx]->prevLeftSteer + ax0 * cmd[CMD_LEFTSTEER].sens * s->deltaTime / (1.0 + cmd[CMD_LEFTSTEER].spdSens * car->pub.speed / 10.0);
 				if (leftSteer > 1.0) leftSteer = 1.0;
 				if (leftSteer < 0.0) leftSteer = 0.0;
 				HCtx[idx]->prevLeftSteer = leftSteer;
@@ -642,7 +632,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 			
 			// normalize ax to 0..1
 			ax0 = (ax0 - cmd[CMD_RIGHTSTEER].min) / (cmd[CMD_RIGHTSTEER].max - cmd[CMD_RIGHTSTEER].min);
-			rightSteer = -SIGN(ax0) * cmd[CMD_RIGHTSTEER].pow * pow(fabs(ax0), cmd[CMD_RIGHTSTEER].sens) / (1.0 + cmd[CMD_RIGHTSTEER].spdSens * car->_speed_x);
+			rightSteer = -SIGN(ax0) * cmd[CMD_RIGHTSTEER].pow * pow(fabs(ax0), cmd[CMD_RIGHTSTEER].sens) / (1.0 + cmd[CMD_RIGHTSTEER].spdSens * car->pub.speed);
 			break;
 		case GFCTRL_TYPE_MOUSE_AXIS:
 			ax0 = mouseInfo->ax[cmd[CMD_RIGHTSTEER].val] - cmd[CMD_RIGHTSTEER].deadZone;
@@ -652,7 +642,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 				ax0 = cmd[CMD_RIGHTSTEER].min;
 			}
 			ax0 = ax0 * cmd[CMD_RIGHTSTEER].pow;
-			rightSteer = - pow(fabs(ax0), cmd[CMD_RIGHTSTEER].sens) / (1.0 + cmd[CMD_RIGHTSTEER].spdSens * car->_speed_x / 10.0);
+			rightSteer = - pow(fabs(ax0), cmd[CMD_RIGHTSTEER].sens) / (1.0 + cmd[CMD_RIGHTSTEER].spdSens * car->pub.speed / 10.0);
 			break;
 		case GFCTRL_TYPE_KEYBOARD:
 		case GFCTRL_TYPE_SKEYBOARD:
@@ -668,7 +658,7 @@ static void common_drive(int index, tCarElt* car, tSituation *s)
 				HCtx[idx]->prevRightSteer = rightSteer = 0;
 			} else {
 				ax0 = 2 * ax0 - 1;
-				rightSteer = HCtx[idx]->prevRightSteer - ax0 * cmd[CMD_RIGHTSTEER].sens * s->deltaTime/ (1.0 + cmd[CMD_RIGHTSTEER].spdSens * car->_speed_x / 10.0);
+				rightSteer = HCtx[idx]->prevRightSteer - ax0 * cmd[CMD_RIGHTSTEER].sens * s->deltaTime/ (1.0 + cmd[CMD_RIGHTSTEER].spdSens * car->pub.speed / 10.0);
 				if (rightSteer > 0.0) rightSteer = 0.0;
 				if (rightSteer < -1.0) rightSteer = -1.0;
 				HCtx[idx]->prevRightSteer = rightSteer;
@@ -1133,11 +1123,20 @@ static void drive_at(int index, tCarElt* car, tSituation *s)
 
 	/* auto shift */
 	if (!HCtx[idx]->manual && !HCtx[idx]->AutoReverseEngaged) {
-		if (car->_speed_x > HCtx[idx]->shiftThld[gear]) {
-			car->_gearCmd++;
-		} else if ((car->_gearCmd > 1) && (car->_speed_x < (HCtx[idx]->shiftThld[gear-1] - 4.0))) {
-			car->_gearCmd--;
+		tdble omega = car->_enginerpmRedLine * car->_wheelRadius(2) * 0.95;
+		tdble shiftThld = 10000.0f;
+		if (car->_gearRatio[gear] != 0) {
+			shiftThld = omega / car->_gearRatio[gear];			
 		}
+
+		if (car->pub.speed > shiftThld) {
+			car->_gearCmd++;
+		} else if (car->_gearCmd > 1) {
+			if (car->pub.speed < (omega / car->_gearRatio[gear-1] - 4.0)) {
+				car->_gearCmd--;
+			}
+		}
+
 		if (car->_gearCmd <= 0) {
 			car->_gearCmd++;
 		}
