@@ -4,7 +4,7 @@
     created              : Mon Apr 17 13:51:00 CET 2000
     copyright            : (C) 2000-2006 by Bernhard Wymann
     email                : berniw@bluewin.ch
-    version              : $Id: lliaw.cpp,v 1.9.2.1 2008/05/31 09:25:28 berniw Exp $
+    version              : $Id$
 
  ***************************************************************************/
 
@@ -169,12 +169,13 @@ static void drive(int index, tCarElt* car, tSituation *situation)
 	tdble b2;							// Brake value for some brake point in front of us.
 	tdble b3;							// Brake value for control (avoid loosing control).
 	tdble b4;							// Brake value for avoiding high angle of attack.
+	tdble b5;							// Brake for the pit;
 	tdble steer, targetAngle, shiftaccel;
 
 	MyCar* myc = mycar[index-1];
 	Pathfinder* mpf = myc->getPathfinderPtr();
 
-	b1 = b2 = b3 = b4 = 0.0;
+	b1 = b2 = b3 = b4 = b5 = 0.0;
 	shiftaccel = 0.0;
 
 	// Update some values needed.
@@ -254,6 +255,13 @@ static void drive(int index, tCarElt* car, tSituation *situation)
 		if (fabs(steer) > 1.0) {
 			steer/=fabs(steer);
 		}
+	} else {
+		// Check if we are almost in the pit to set brake to the max to avoid overrun.
+		tdble dl, dw;
+		RtDistToPit(car, myTrackDesc->getTorcsTrack(), &dl, &dw);
+		if (dl < 1.0f) {
+			b5 = 1.0f;
+		}
 	}
 
 	// Try to control angular velocity with a D (differential) controller.
@@ -338,7 +346,9 @@ static void drive(int index, tCarElt* car, tSituation *situation)
 	float maxForce = weight + myc->ca*myc->MAX_SPEED*myc->MAX_SPEED;
 	float force = weight + myc->ca*myc->getSpeedSqr();
 	brake = brake*MIN(1.0, force/maxForce);
-
+	if (b5 > 0.0f) {
+		brake = b5;
+	}
 
 	// Gear changing.
 	if (myc->tr_mode == 0) {
