@@ -4,7 +4,7 @@
     created              : Wed Mar 21 23:06:29 CET 2001
     copyright            : (C) 2001 by Eric Espi
     email                : Eric.Espie@torcs.org
-    version              : $Id: joystickconfig.h,v 1.3 2003/11/08 16:37:18 torcs Exp $
+    version              : $Id$
 
  ***************************************************************************/
 
@@ -20,13 +20,13 @@
 /** @file    
     		
     @author	<a href=mailto:torcs@free.fr>Eric Espie</a>
-    @version	$Id: joystickconfig.h,v 1.3 2003/11/08 16:37:18 torcs Exp $
+    @version	$Id$
 */
 
 #ifndef _JOYSTICKCONFIG_H_
 #define _JOYSTICKCONFIG_H_
 
-extern void *JoyCalMenuInit(void *prevMenu, tCmdInfo *cmd, int maxcmd);
+extern void *JoyCalMenuInit(void *prevMenu, tCmdInfo *cmd, int maxcmd, void *parmHandle, const char* driverSection);
 
 #endif /* _JOYSTICKCONFIG_H_ */ 
 
