@@ -18,7 +18,7 @@
  
      For further information visit http://plib.sourceforge.net
 
-     $Id: grloadac.cpp,v 1.20.2.2 2011/12/29 11:59:43 berniw Exp $
+     $Id$
 */
 
 #include <plib/ssg.h>
@@ -156,12 +156,12 @@ static void skip_quotes ( char **s )
       t++ ;
 
     if ( *t != '\"' )
-      ulSetError ( UL_WARNING, "ac_to_gl: Mismatched double-quote ('\"') in '%s'", *s ) ;
+      ulSetError ( UL_WARNING, "ac_to_gl: Mismatched double-quote ('\"') in '%900s'", *s ) ;
 
     *t = '\0' ;
   }
   else
-    ulSetError ( UL_WARNING, "ac_to_gl: Expected double-quote ('\"') in '%s'", *s ) ;
+    ulSetError ( UL_WARNING, "ac_to_gl: Expected double-quote ('\"') in '%900s'", *s ) ;
 }
 
 
@@ -180,7 +180,7 @@ static int search ( Tag *tags, char *s )
       return (*(tags[i].func))( s ) ;
     }
 
-  ulSetError ( UL_FATAL, "ac_to_gl: Unrecognised token '%s' (%d)", s , strlen(s)) ;
+  ulSetError ( UL_WARNING, "ac_to_gl: Unrecognised token '%900s' (%d)", s , strlen(s)) ;
 
   return 0 ;  /* Should never get here */
 }
@@ -679,9 +679,19 @@ static int do_numvert  ( char *s )
   t1tab = new sgVec2 [ nv ] ;
   t2tab = new sgVec2 [ nv ] ;
   t3tab = new sgVec2 [ nv ] ;
+  
+  if (vertlist != NULL) {
+	  ssgDeRefDelete(vertlist);
+  }
+  if (striplist != NULL) {
+	  ssgDeRefDelete(striplist);
+  }
+  
   vertlist=new ssgIndexArray ();
+  vertlist->ref();
   striplist=new ssgIndexArray ();
-
+  striplist->ref();
+  
   for ( int i = 0 ; i < nv ; i++ )
   {
     FGETS ( buffer, 1024, loader_fd ) ;
@@ -1204,7 +1214,7 @@ static ssgEntity *myssgLoadAC ( const char *fname, const ssgLoaderOptions* optio
 
   if ( loader_fd == NULL )
   {
-    ulSetError ( UL_WARNING, "ssgLoadAC: Failed to open '%s' for reading", filename ) ;
+    ulSetError ( UL_WARNING, "ssgLoadAC: Failed to open '%900s' for reading", filename ) ;
     return NULL ;
   }
 
@@ -1234,7 +1244,7 @@ static ssgEntity *myssgLoadAC ( const char *fname, const ssgLoaderOptions* optio
       if ( ! ulStrNEqual ( s, "AC3D", 4 ) )
       {
         FCLOSE ( loader_fd ) ;
-        ulSetError ( UL_WARNING, "ssgLoadAC: '%s' is not in AC3D format.", filename ) ;
+        ulSetError ( UL_WARNING, "ssgLoadAC: '%900s' is not in AC3D format.", filename ) ;
         return NULL ;
       }
     }
